//
//  KMRequest.h
//  Koupon
//
//  Created by Brian Reinhart on 9/24/13.
//  Copyright (c) 2013 Brian Reinhart. All rights reserved.
//

#import <Foundation/Foundation.h>

/** This class is returned from every Koupon call.
 
 There is no reason to manually initialize this class unless you would like to manually build a Koupon API request.
 
    Koupon* kouponSDK = [[Koupon alloc] init];
 
    //Post data to send to createConsumerIdentity API
    NSMutableDictionary* data = [[NSMutableDictionary alloc] init];
    [data setObject:@"blahblah@email.com" forKey:@"email"];
 
    //Get the KMRequest object with the signed URL.
    KMRequest* request = [kouponSDK createConsumerIdentity:data withOptionalQueryData:nil];
    [request print]; //prints out all of the properties of the KMRequest
 
 */

@interface KMRequest : NSObject {
    void (^_successHandler)(NSMutableDictionary* jsonResult);
    void (^_failureHandler)(NSMutableDictionary* jsonResult);
    NSMutableData* asyncData;
}

/**---------------------------------------------------------------------------------------
 * @name Public Properties
 *  ---------------------------------------------------------------------------------------
 */

/** The signed url */
@property NSString* url;
/** The REST method for communicating to the Server */
@property NSString* method;
/** An NSDictionary containing all of the body parameters.  This will be empty for all GET methods */
@property NSDictionary* bodyDict;
/** The serialized JSON string representing the bodyDict dictionary */
@property NSString* bodyString;

/**---------------------------------------------------------------------------------------
 * @name Initialization
 *  ---------------------------------------------------------------------------------------
 */

/** Initialize a new KMRequest class
 
 There is really no need to do this, as you will always receive the KMRequest object from a Koupon API call
 @param NSString A url containing all of the required query parameters and a signed authSignature appended to the end
 @param NSDictionary A dictionary containing all of the POST/PUT request body parameters.
 @return KMRequest
 */
-(id) init: (NSString*) theUrl method: (NSString*) theMethod body:(NSDictionary *) theBody;

/**---------------------------------------------------------------------------------------
 * @name Helper Functions
 *  ---------------------------------------------------------------------------------------
 */

/** Prints out all of the properties of this KMRequest class
 @return void
 */
- (void) print;
/** Executes the request and returns a dictionary of the results
 
 The results dictionary is a key value pair representing all the JSON results as well as an httpCode and Raw key to store the http code and the raw response data.
 @return NSMutableDictionary a dictionary representation of the JSON response
 */
- (NSMutableDictionary*) sendRequest;

/** Executes the request asynchronously and calls the callback block once complete
 
 The callback passes a results dictionary containing key value pair representing all the JSON results as well as an httpCode and Raw key to store the http code and the raw response data.
 @param block Block to call on success
 @param block Block to call on failure
 @return void
 */
- (void) sendRequestAsyncOnSuccess: (void(^)(NSMutableDictionary*))success OnFailure: (void(^)(NSMutableDictionary*))failure;

@end
